@@ -20,11 +20,13 @@ from __future__ import division, absolute_import, print_function
 import os
 import shutil
 import sqlite3
+import unittest
+from six import assertRaisesRegex
 
 from test import _common
-from test._common import unittest
 from beets import dbcore
 from tempfile import mkstemp
+import six
 
 
 # Fixture: concrete database and model classes. For migration tests, we
@@ -298,9 +300,9 @@ class ModelTest(unittest.TestCase):
         self.assertNotIn('flex_field', model2)
 
     def test_check_db_fails(self):
-        with self.assertRaisesRegexp(ValueError, 'no database'):
+        with assertRaisesRegex(self, ValueError, 'no database'):
             dbcore.Model()._check_db()
-        with self.assertRaisesRegexp(ValueError, 'no id'):
+        with assertRaisesRegex(self, ValueError, 'no id'):
             TestModel1(self.db)._check_db()
 
         dbcore.Model(self.db)._check_db(need_id=False)
@@ -312,13 +314,13 @@ class ModelTest(unittest.TestCase):
     def test_computed_field(self):
         model = TestModelWithGetters()
         self.assertEqual(model.aComputedField, 'thing')
-        with self.assertRaisesRegexp(KeyError, u'computed field .+ deleted'):
+        with assertRaisesRegex(self, KeyError, u'computed field .+ deleted'):
             del model.aComputedField
 
     def test_items(self):
         model = TestModel1(self.db)
         model.id = 5
-        self.assertEqual({('id', 5), ('field_one', None)},
+        self.assertEqual({('id', 5), ('field_one', 0)},
                          set(model.items()))
 
     def test_delete_internal_field(self):
@@ -328,7 +330,7 @@ class ModelTest(unittest.TestCase):
             model._db
 
     def test_parse_nonstring(self):
-        with self.assertRaisesRegexp(TypeError, u"must be a string"):
+        with assertRaisesRegex(self, TypeError, u"must be a string"):
             dbcore.Model._parse(None, 42)
 
 
@@ -347,9 +349,9 @@ class FormatTest(unittest.TestCase):
 
     def test_format_flex_field_bytes(self):
         model = TestModel1()
-        model.other_field = u'caf\xe9'.encode('utf8')
+        model.other_field = u'caf\xe9'.encode('utf-8')
         value = model.formatted().get('other_field')
-        self.assertTrue(isinstance(value, unicode))
+        self.assertTrue(isinstance(value, six.text_type))
         self.assertEqual(value, u'caf\xe9')
 
     def test_format_unset_field(self):
